/* SAMHAIN file system integrity testing                                   */
/* Copyright (C) 1999 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>


#include "config_xor.h"

#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif


#include "samhain.h"
#include "sh_error.h"
#include "sh_unix.h"
#include "sh_utils.h"
#include "sh_files.h"
#include "sh_getopt.h"
#include "sh_readconf.h"
#include "sh_hash.h"
#include "sh_mail.h"
#include "sh_tiger.h"
#include "sh_mem.h"
#include "sh_forward.h"

#ifdef SH_STEALTH

#ifndef SH_MAX_GLOBS
#define SH_MAX_GLOBS 32
#endif
char * globber(char * str)
{
  register int i, j;
  static int  count = -1;
  static char glob[SH_MAX_GLOBS][128];

  ++count; if (count > (SH_MAX_GLOBS-1) ) count = 0;
  j = strlen(str);
  if (j > 127) j = 127;

  for (i = 0; i < j; ++i)
    {
      if (str[i] != '\n' && str[i] != '\t') 
	glob[count][i] = str[i] ^ XOR_CODE;
      else
	glob[count][i] = str[i];
    }
  glob[count][j] = '\0';
  return glob[count];
}
#endif


sh_struct sh;
sh_key_t  *skey;

void sh_init ()
{

  sh.checkSum = SH_CHECK_NONE;

  strlcpy (sh.localhost,  _("localhost"),        SH_MINIBUF+1);
  strlcpy (sh.address,    _("nobody@nowhere"),   256+1);
  strlcpy (sh.timeserver, DEFAULT_TIMESERVER,    256+1);

  sh.looptime     = 60;
  sh.modus        = 0;
  sh.isserver     = BAD;
  sh.locked       = BAD;

  sh.fileCheck.alarm_last     = 0;
  sh.fileCheck.alarm_interval = 600; /* ten minutes */

  sh.mailTime.alarm_last     = 0;
  sh.mailTime.alarm_interval = 86400;

  sh.mailNum.alarm_last      = 0;
  sh.mailNum.alarm_interval  = 10;

  sh.real_user[0] = '\0';

  sh.start_time              = (unsigned long) time (NULL);

  skey = (sh_key_t *) malloc (sizeof(sh_key_t));
  if (skey == NULL) 
    {
      perror(_("sh_init"));
      _exit (EXIT_FAILURE);
    }
  skey->mlock_failed = GOOD;

  sh_unix_mlock();

  return;
}


#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
#include <sys/mman.h>
#endif

void exit_handler()
{
#ifdef MEM_DEBUG
#if defined(SH_WITH_CLIENT) || defined(SH_STANDALONE)
  sh_files_deldirstack ();
  sh_files_delfilestack ();
  sh_hash_hashdelete();
#endif
#if defined(SH_WITH_SERVER)
  sh_forward_free_all ();
#endif
  sh_mem_stat();
#endif

#if defined(SH_WITH_SERVER) && !defined(SH_WITH_CLIENT)
  sh_error_handle (SH_ERR_STAMP, PLACE 
		   NULL,
		   _("Exit yule"));
#else
  sh_error_handle (SH_ERR_STAMP, PLACE 
		   NULL,
		   _("Exit samhain"));
#endif
  sh_unix_unlock();
  memset (skey, '\0', sizeof(sh_key_t));
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
  (void) munlock( (char *) skey, sizeof(sh_key_t) );
#endif
  
  _exit(EXIT_SUCCESS);
  return;
}

int main(int argc, char * argv[])
{

  unsigned long tcurrent;
  
  (void) atexit(exit_handler); 
  
  /* --------  INIT  --------    
   */

  (void) sh_unix_droppriv (); /* do not honour SUID settings */

  /* restrict error logging to stderr
   */
  sh_error_only_stderr (GOOD);

  /* default values
   */
  sh_init ();

  /* hostname
   */
  sh_unix_localhost();

  /* read the configuration file
   */
  sh.opts = 0;
  (void) sh_readconf_read ();

  /* read the command line
   */
  sh.opts = 1;
  (void) sh_getopt_get (argc, argv);
  sh.opts = 0;

  /* initialize signal handling etc
   */
  if (sh.modus == 1) /* go daemon */ 
    sh_error_only_stderr (BAD);
  if (sh_unix_init(sh.modus) == -1) {
    fprintf (stderr, _("Error initializing the application\n"));
    exit(EXIT_FAILURE);
  }

  /* checksum of database
   */
  if (sh.checkSum == SH_CHECK_CHECK) {
    strlcpy(sh.filedata_hash,
	    sh_tiger_hash (DEFAULT_DATA_FILE, TIGER_FILE, 0), 
	    51);
  }

  /* username
   */
  if (GOOD == sh_unix_getUser()) 
    {
      /* enable full error logging 
       */
      sh_error_only_stderr (BAD);
#if defined(SH_WITH_SERVER) && !defined(SH_WITH_CLIENT)
      sh_error_handle (SH_ERR_STAMP, PLACE 
		       _("Start yule"),
		       _("Start yule by user: %s with home directory %s"), 
		       sh.user, sh.home);
#else
      sh_error_handle (SH_ERR_STAMP, PLACE 
		       _("Start samhain"),
		       _("Start samhain by user: %s with home directory %s"), 
		       sh.user, sh.home);
      if (sh.checkSum == SH_CHECK_CHECK) {
	sh_error_handle (SH_ERR_FATAL, PLACE
			 NULL,
			 _("Database signature: %s"),
			 sh.filedata_hash);
      }
#endif
    } 
  else 
    {
      sh_error_handle (SH_ERR_FATAL, PLACE
		       NULL,
		       _("Cannot obtain your user info, abort"));
      exit (EXIT_FAILURE);
    }
 
  if (skey->mlock_failed == BAD)
    sh_error_handle (SH_ERR_WARN, PLACE 
		     NULL,
		     _("Using insecure memory"));

  /* timer
   */
  tcurrent                   = (unsigned long) time (NULL);
  sh.mailTime.alarm_last     = tcurrent;


  /* -------  run as log server -------------
   */
#ifdef SH_WITH_SERVER
  if (sh.isserver == GOOD)
    { 
      sh_receive();
      exit (EXIT_SUCCESS);
    }
#endif

#if defined (SH_WITH_CLIENT) || defined (SH_STANDALONE) 
    

  /*  --------  MAIN LOOP  ---------
   */

  while (1) {

    tcurrent                   = (unsigned long) time (NULL);

    /* see whether its time to check files
     */
    if (sh.checkSum != SH_CHECK_NONE &&
	tcurrent - sh.fileCheck.alarm_last >= sh.fileCheck.alarm_interval) {
      sh_dirs_chk  (); 
      sh_files_chk ();
      sh_error_handle (SH_ERR_STAMP, PLACE
		     NULL, _("File check completed."));
      sh.fileCheck.alarm_last = (unsigned long) time (NULL);
    }
    
    /* no loop if not deamon 
     */
    if (sh.modus == 0) break; 

    /* see whether its time to send mail
     */
    if (tcurrent - sh.mailTime.alarm_last >= sh.mailTime.alarm_interval) {
      (void) sh_mail_msg (sh.address, _("---- ALIVE ----"));
      sh.mailTime.alarm_last = (unsigned long) time (NULL);
    }

    sh_hash_hashdelete(); 
    (void) sleep ( (unsigned int) sh.looptime );

#ifdef MEM_DEBUG
    sh_mem_check();
#else
    sh_error_handle (SH_ERR_STAMP, PLACE
		     NULL, _("---- TIMESTAMP ----"));
#endif
  }

  /*   ------  END  -----------
   */

  if (sh.mailNum.alarm_last > 0) {
    (void)sh_mail_msg (sh.address, NULL);
  }

#endif

  exit (EXIT_SUCCESS);
}



