/* SAMHAIN file system integrity testing                                   */
/* Copyright (C) 1999 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include <stdio.h>     
#include <stdlib.h>     
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <syslog.h>

#include "config_xor.h"

#ifdef HAVE_MEMORY_H
#include <memory.h>
#endif

#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
#include <sys/mman.h>
#endif

#if defined(__linux__)
#include <paths.h>
#endif
#ifndef _PATH_CONSOLE
#define _PATH_CONSOLE   "/dev/console"
#endif

#include "samhain.h"
#include "sh_error.h"
#include "sh_utils.h"
#include "sh_unix.h"
#include "sh_tiger.h"
#include "sh_mail.h"
#include "sh_forward.h"



#if !defined (VA_COPY)
#if defined (__GNUC__) && defined (__PPC__) && (defined (_CALL_SYSV) || defined (_WIN32))
#define VA_COPY(ap1, ap2)     (*(ap1) = *(ap2))
#elif defined (VA_COPY_AS_ARRAY)
#define VA_COPY(ap1, ap2)     memmove ((ap1), (ap2), sizeof (va_list))
#else /* va_list is a pointer */
#define VA_COPY(ap1, ap2)     ((ap1) = (ap2))
#endif
#endif 


ShErrLevel  ShDFLevel[SH_ERR_T_END];

struct  errFlagsStruct {
  int           debug;
  int           HaveLog;
  ShErrLevel    loglevel;
  ShErrLevel    loglevel_temp;
  ShErrLevel    printlevel;
  ShErrLevel    maillevel;
  ShErrLevel    exportlevel;
  ShErrLevel    sysloglevel;
  ShErrLevel    severity;
  char          errmsg[SH_MINIBUF+2];
  char          errfile[PATH_MAX+2];
} errFlags;

static int  sh_error_init ();
static int  sh_error_string (char *message,  ShErrLevel severity,
			     const char *sourcefile, long sourceline, 
			     const char *fmt, va_list vl);
static int  sh_error_print (char *message);
static int  sh_error_log (char *message);
static int  sh_error_syslog (ShErrLevel  severity, char *message);


static int  IsInitialized = BAD;

static int  OnlyStderr    = GOOD; /* only log to stderr */


/*********************************************
 *  utility functions for verifying entries
 *********************************************/

int sh_error_verify (char * s)
{
  fprintf (stdout, _("HASH : |%s|\n"), 
	   sh_tiger_hash (s, TIGER_DATA, strlen(s)));
  fprintf (stdout, _("ORIG : |%s|\n"), s);
  fflush(stdout);
  _exit (EXIT_SUCCESS);
  return 0;
}

int sh_error_logverify (char * s)
{
  int fd;
  int len;
  int count =  0;
  int start = -1;
  char * buf;
  char * bufc;
  char signature[64];
  char key[KEY_MAX+1];

  (void) sh_unix_droppriv (); /* do not honour SUID settings */

  sh_error_logoff();

  if (s == NULL || strlen(s) >= PATH_MAX) 
    exit (EXIT_FAILURE);

  /* open the file, then check it 
   */
  if ( (fd = sh_unix_fileopen (s, 'r', 
			       SH_ISFILE, SH_ERR_FATAL)) == (-1))
    {
      fprintf(stderr, _("Could not open file %s\n"), s);
      exit (EXIT_FAILURE);
    }

  buf  = SH_ALLOC( (unsigned long)(2*BUFSIZ+1));
  bufc = SH_ALLOC( (unsigned long)(2*BUFSIZ+1));

  while (1) {

    /* get the log message
     */
    if (sh_unix_getline (fd, buf, (2*BUFSIZ)) < 0) break;
    len = (int) strlen(buf);

#ifdef SH_STEALTH
    if (0 == strncmp (buf, N_("[SOF]"), 5)) 
#else
    if (0 == strncmp (buf, _("[SOF]"),  5)) 
#endif
      {
	/* found start of audit trail, read first line */
	start = 1;
	(void) sh_unix_getline (fd, buf, (2*BUFSIZ));
	len = (int) strlen(buf);
	++count;
      }
    else if (buf[0] == '\n')
      {
	/* a newline */
	++count;
	continue;
      }
    else if (start == 0)
      {
	/* we are inside an audit trail */
	++count;
      }
    else
      {
	/* no start-of-file found yet */
	continue;
      }

    /* check for continuation line
     */
    while (1 == 1)
      {
	(void) sh_unix_getline (fd, bufc, (2*BUFSIZ));
	++count;
	if (bufc[0] == ' ') /* a continuation line */
	  {
	    strlcat(buf, "\n", 2*BUFSIZ+1);  /* add the newline */
	    ++len;
	    strlcat(buf, bufc, 2*BUFSIZ+1);
	    len += (int) strlen(bufc);
	  }
	else
	  {
	    strlcpy(signature, bufc, 64);
	    break;
	  }
      }

    if (start == 1) {      /* get starting key from cl */
      fprintf(stdout, _("New audit trail, enter key: "));
      fflush(stdout); key[0] = '\0';
      while (strlen(key) < KEY_MAX ) { 
	fgets(key, KEY_MAX+1, stdin); 
      }
      sh_util_encode(key, buf, 1);
      start = 0;
    } else {               /* iterate key */
      strlcpy (key, 
	       sh_tiger_hash (key, TIGER_DATA, KEY_MAX), 
	       51);
    }

    strlcat ( buf, key, 2*BUFSIZ + 1);

#ifdef SH_STEALTH
    sh_unix_xor_code (signature, strlen(signature));
#endif


    if (strncmp (signature, 
		 sh_tiger_hash (buf, TIGER_DATA, strlen(buf)),
		 50) != 0) {
      buf[len] = '\0';    /* do not print out the key */
#ifdef SH_STEALTH
      sh_unix_xor_code (buf, strlen(buf));
#endif
      fprintf (stdout, _("FAILED (%5d): %s\n"), count-1, buf);

    } else { 
      buf[len] = '\0';    /* do not print out the key */
#ifdef SH_STEALTH
      sh_unix_xor_code (buf, strlen(buf));
#endif
      fprintf (stdout, _("passed (%5d): %s\n"), count-1, buf);
    }
  }

  sh_unix_fileclose (fd);
  SH_FREE(buf);

  fflush(stdout);

  _exit (EXIT_SUCCESS);
  return 0;
}

/*********************************************
 *  end utility functions
 *********************************************/

void sh_error_only_stderr (int flag)
{
  OnlyStderr    = flag;
  return;
}

char * sh_error_message (int tellme)
{

#ifdef HAVE_STRERROR
  return strerror(tellme);
#else

#ifdef EACCES
    if (tellme == EACCES)  return _("Permission denied.");
#endif
#ifdef EAGAIN
    if (tellme == EAGAIN)  return _("Try again.");
#endif
#ifdef EBADF
    if (tellme == EBADF)   return _("File descriptor in bad state.");
#endif
#ifdef EEXIST
    if (tellme == EEXIST)  return _("File exists.");
#endif
#ifdef EFAULT
    if (tellme == EFAULT)  return _("Bad address.");
#endif
#ifdef EINVAL
    if (tellme == EINVAL)  return _("Invalid argument.");
#endif
#ifdef EISDIR
    if (tellme == EISDIR)  return _("Is a directory.");
#endif
#ifdef EINTR
    if (tellme == EINTR)   return _("System call was interrupted.");
#endif
#ifdef EIO
    if (tellme == EIO)     return _("Low-level I/O error.");
#endif
#ifdef ELOOP
    if (tellme == ELOOP)   return _("Too many symbolic links encountered.");
#endif
#ifdef EMFILE
    if (tellme == EMFILE)  return _("Too many open files.");
#endif
#ifdef ENAMETOOLONG
    if (tellme == ENAMETOOLONG) 
                           return _("File name too long."); 
#endif
#ifdef ENFILE
    if (tellme == ENFILE)  return _("File table overflow.");
#endif
#ifdef ENOENT
    if (tellme == ENOENT)  return _("File does not exist.");
#endif
#ifdef ENOMEM
    if (tellme == ENOMEM)  return _("Out of memory.");
#endif
#ifdef ENOSPC
    if (tellme == ENOSPC)  return _("No space on device.");
#endif
#ifdef ENOTDIR
    if (tellme == ENOTDIR) return _("Not a directory.");
#endif
#ifdef ENOTSOCK
    if (tellme == ENOTSOCK) return _("Not a socket.");
#endif
#ifdef EOPNOTSUPP
    if (tellme == EOPNOTSUPP) return _("Socket is not of type SOCK_STREAM.");
#endif
#ifdef EPIPE
    if (tellme == EPIPE)   return _("No read on pipe.");
#endif
#ifdef EROFS
    if (tellme == EROFS)    return _("Read-only file system.");
#endif
#ifdef ETXTBSY
    if (tellme == ETXTBSY) return _("Text file busy.");
#endif
#ifdef EWOULDBLOCK
    if (tellme == EWOULDBLOCK) 
      return _("No connections on non-blocking socket.");
#endif
    return _("Unknown error");
#endif /* ifndef HAVE_STRERROR */
}


/* switch off file log
 */
void sh_error_logoff()
{
  errFlags.HaveLog = BAD;
  return;
}

/* switch on file log 
 */
void sh_error_logrestore()
{
  errFlags.HaveLog = GOOD;
  return;
}

/* set severity levels
 */
int sh_error_set_iv (int iv, char *  flag_s)
{
  int level;

  level = atoi (flag_s);
  if (level <= SH_ERR_T_START || level >= SH_ERR_T_END) {
    sh_error_handle (SH_ERR_ERR, PLACE
		     _("Set severity - unknown level"),
		     _("Set severity - unknown level %d"),
		     (int) level);
    return EXIT_FAILURE;
  }

  if (iv > SH_ERR_T_START && iv < SH_ERR_T_END) {
    ShDFLevel[iv] =  level;
  } else {
    sh_error_handle (SH_ERR_ERR, PLACE
		     _("Set severity - unknown resource"),
		     _("Set severity - unknown resource %d"),
		     (int) iv);
    return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}

int sh_error_set_level(char * str_s, ShErrLevel *facility)
{
  ShErrLevel flag = -1;

  if (IsInitialized == BAD) (void) sh_error_init();

  if (str_s != NULL) 
    flag = (int)(atof(str_s));

  if (flag >= 0 && flag <= SH_ERR_NOT) {
    *facility = flag;
  } else {
    sh_error_handle (SH_ERR_ERR, PLACE
		     NULL,
		     _("Invalid threshold: %ld"),
		     flag);
  }

  return (EXIT_SUCCESS);
}

#ifdef SH_WITH_CLIENT
/* set severity for TCP export
 */
int sh_error_setexport(char *  str_s)
{
  return (sh_error_set_level(str_s, &errFlags.exportlevel));
}
#endif

/* set severity for printing
 */
int sh_error_setprint(char *  str_s)
{
  return (sh_error_set_level(str_s, &errFlags.printlevel));
}


/* set level for error logging
 */
int sh_error_setlog(char * str_s)
{
  return (sh_error_set_level(str_s, &errFlags.loglevel));
}


/* set severity for syslog
 */
int sh_error_set_syslog (char * str_s)
{
  return (sh_error_set_level(str_s, &errFlags.sysloglevel));
}

/* set severity for mailing
 */
int sh_error_setseverity (char * str_s)
{
  return (sh_error_set_level(str_s, &errFlags.maillevel));
}

  
/* set debug level
 */
int sh_error_setdebug (char * debug_s)
{
  int debug;

  if (IsInitialized == BAD) sh_error_init();

  if (debug_s == NULL) 
    debug = 0;
  else
    debug = (int)(atof(debug_s));

  if (debug >= 0 && debug < 3) {
    errFlags.debug = debug;
    return (EXIT_SUCCESS);
  } else {
    sh_error_handle (SH_ERR_ERR, PLACE
		     NULL,
		     _("Unsupported debug level: %ld"),
		     debug);
  }
  return (EXIT_FAILURE);
}
  
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
#include <sys/mman.h>
#endif

/* out-of-memory errors are handled by static memory
 */
static int oom          = GOOD;


/**********************************************************
 **********************************************************
 *
 * --------  MAIN ERROR HANDLING FUNCTION -----------------
 *
 *
 * this function should be called to report an error
 *
 ********************************************************** 
 **********************************************************/
void sh_error_handle (ShErrLevel  severity, 
		     char * sourcefile,
#ifdef MEM_DEBUG
		     long sourceline,
#endif 
		     const char *fmt1, const char *fmt2, ...)
{
#ifndef MEM_DEBUG
  long          sourceline = 0;     /* dummy line         */
#endif
  va_list       vl;                 /* argument list      */
  char        * message = NULL;     /* pointer to message */

  /* block a facility for errors generated
   * within that facility
   */
  static int print_block  = 0;
  static int mail_block   = 0;
  static int syslog_block = 0;
  static int log_block    = 0;
#ifdef SH_WITH_CLIENT
  static int export_block = 0;
#endif

  /* out-of-memory errors are handled by static memory
   */
  static int oom          = GOOD;

  /* initialize to default values
   */
  if (IsInitialized == BAD) 
    sh_error_init();


  /* block unwanted stuff
   */
  if ((int)(severity) >= (int)(SH_ERR_MAX)) 
    {
      sh_error_handle (SH_ERR_ERR, PLACE
		       _("Internal error: error message with "\
			 "invalid severity"),
		       _("Internal error: severity in error message "\
			 "is > SH_ERR_MAX"));
      return;
    }
  else if (
	   (
	    (int)(severity) < (int)(errFlags.printlevel)    &&
	    (int)(severity) < (int)(errFlags.loglevel)      &&
	    (int)(severity) < (int)(errFlags.sysloglevel)   &&
#ifdef SH_WITH_CLIENT
	    (int)(severity) < (int)(errFlags.exportlevel)   &&
#endif
	    (int)(severity) < (int)(errFlags.maillevel)
	    ) || (severity == SH_ERR_NOT)
	   )
    return;
  else if ( severity == SH_ERR_FAIL && oom == BAD )
    return;
  else if (fmt2 == NULL) 
    {
      sh_error_handle (SH_ERR_ERR, PLACE
		       _("Internal error: message with invalid format"),
		       _("Internal error: fmt2 in error message is NULL"));
      return;
    }

  /* handle out-of-memory-errors specially
   */
  if ( severity == SH_ERR_FAIL )
    {
      oom      = BAD;                 /* set the flag      */
      message  = &errFlags.errmsg[0]; /* use static memory */
      severity = SH_ERR_FATAL;        /* reset severity    */
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	mlock( message, SH_MINIBUF+2);
#endif
    }
  else
    {
      message = SH_ALLOC(BUFSIZ+2);
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	mlock( message, BUFSIZ+2);
#endif
    }
  message[0] = '\0';

  /* format the log message with timestamp etc
   */
  va_start (vl, fmt2);
  if (errFlags.debug > 0 || fmt1 == NULL) 
    sh_error_string (message, severity, 
		     sourcefile, sourceline, fmt2, vl);
  else 
    sh_error_string (message, severity, 
		     sourcefile, sourceline, fmt1, vl);
  va_end (vl);

  /* log to stderr
   */
  if ((int)(severity) >= (int)(errFlags.printlevel))
    { 
      if (print_block == 0) 
	{
	  print_block = 1;
	  sh_error_print (message);
	  print_block = 0;
	}
    }

  if (OnlyStderr == BAD)  /* full error logging enabled */
    {

      /* log to syslog
       */
      if ((int)(severity) >= (int)(errFlags.sysloglevel) && 
	  (int)(severity) <  (int)(SH_ERR_NOT) ) /* don't log inet to syslog */
	{
	  if (syslog_block == 0)
	    {
	      syslog_block = 1;
	      sh_error_syslog (severity, message);
	      syslog_block = 0;
	    }
	}

      /*****************
       * optionally include client code for TCP forwarding
       * to log server
       *****************/
#ifdef SH_WITH_CLIENT
      /* export by TCP
       */
      if ((int)(severity) >= (int)(errFlags.exportlevel) && 
	  sh.isserver != GOOD                            &&
	  (int)(severity) <  (int)(SH_ERR_NOT) ) /* don't log inet to export */
	{
	  if (export_block == 0)
	    {
	      export_block = 1;
	      sh_forward (message);
	      export_block = 0;
	    }
	}
#endif
      
      /* log to mail
       */
      if ((int)(severity) >= (int)(errFlags.maillevel) &&
	  (int)(severity) <  (int)(SH_ERR_NOT) ) /* don't log inet to email */
	{
	  if (mail_block == 0)
	    {
	      mail_block = 1;
	      if (severity != SH_ERR_FATAL) {
		sh_mail_pushstack (message);
	      } else {
		sh_mail_msg (sh.address, message);
	      }
	      mail_block = 0;
	    }
	}

      /* log to logfile
       */
      if ((int)(severity) >= (int)(errFlags.loglevel) &&
	  (errFlags.HaveLog != BAD) &&  /* temporary switched off */
	  (severity != SH_ERR_NOT) /* paranoia */
	  ) 
	{
	  if (log_block == 0)
	    {
	      log_block = 1;
	      sh_error_log (message);
	      log_block = 0;
	    }
	}

    }

  /* unlock
   */
  if (oom == BAD) 
    {
      memset ( message, '\0', SH_MINIBUF+2 );
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	munlock( message, SH_MINIBUF+2 );
#endif
      oom = GOOD;
    }
  else
    {
      memset ( message, '\0', BUFSIZ+2 );
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	munlock( message,      BUFSIZ+2 );
#endif
      SH_FREE(message);
    }

  return;
}


/* -------------------------  
 *
 * private functions below
 *
 * -------------------------
 */

/* print an error  into string
 */
static int sh_error_string (char *errmsg, ShErrLevel severity,
			    const char *sourcefile, long sourceline, 
			    const char *fmt, va_list vl)
{
#ifdef MEM_DEBUG
  static char   theline[68];
#endif
  static char   errhead[81+17+9];
  static char   errshort[64+2];
#if !defined(HAVE_VSNPRINTF) || defined(HAVE_BROKEN_VSNPRINTF)
  va_list       vl2;
#endif
  int           total;
  char        * errbody;
  int           isshort;

  total   = 0;
  isshort = 0;

  /* header of error message
   */
  if      (severity == SH_ERR_INET)
    strlcpy (errhead, _("<TCP>  : "), 11);
  else if (severity == SH_ERR_ALL)
    strlcpy (errhead, _(";  "),       11);
  else if (severity == SH_ERR_INFO)
    strlcpy (errhead, _("INFO   : "), 11);
  else if (severity == SH_ERR_WARN)
    strlcpy (errhead, _("WARN   : "), 11);
  else if (severity == SH_ERR_STAMP)
    strlcpy (errhead, _("TIME   : "), 11);
  else if (severity == SH_ERR_ERR)
    strlcpy (errhead, _("ERROR  : "), 11);
  else if (severity == SH_ERR_SEVERE)
    strlcpy (errhead, _("SEVERE : "), 11);
  else if (severity == SH_ERR_FATAL)
    strlcpy (errhead, _("URGENT : "), 11);
  else {
    strlcpy (errhead, _("???    : "), 11);
  }


#ifdef MEM_DEBUG
  /* optional source file and line if extended debug facility in use
   */
  if (errFlags.debug > 1) {
    sprintf(theline, " L%ld ",   sourceline);         /* known to fit  */
    strlcat(errhead, sourcefile, sizeof(errhead));
    strlcat(errhead, theline,    sizeof(errhead));
  }
#endif


  /* body of error message -- do not use VA_COPY if not required,
   * in case we are 'Out of memory'
   */
  if ( (NULL == strchr(fmt, '%')) && (strlen(fmt) <= 64)) 
    {
      isshort = 1;
      strlcpy (errshort, fmt, sizeof(errshort));
      errbody = &errshort[0];
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	mlock( errbody, sizeof(errshort));
#endif
    }
 


  /* use VA_COPY 
   */
  else 
    {
      isshort = 0;
      errbody = SH_ALLOC(BUFSIZ+2);
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
      if (skey->mlock_failed == GOOD) 
	mlock( errbody, BUFSIZ+2);
#endif
#if defined(HAVE_VSNPRINTF) && !defined(HAVE_BROKEN_VSNPRINTF)
      vsnprintf (errbody, (BUFSIZ-sizeof(errhead)), fmt, vl);
#else
      VA_COPY (vl2, vl);              /* save the argument list           */
      total = sh_util_printf_maxlength (fmt, vl);
      if (total < (BUFSIZ-sizeof(errhead))) 
	vsprintf (errbody, fmt, vl2); /* program has checked that it fits */
      else 
	strlcpy (errbody, fmt, (BUFSIZ-sizeof(errhead)));
      va_end(vl2);
#endif  
    }

  /* oom == BAD means we use static memory
   */
  if (oom == BAD) 
    {
      strlcpy (errmsg, errhead,          SH_MINIBUF+1);
      strlcat (errmsg, " ",              SH_MINIBUF+1);
      strlcat (errmsg, errbody,          SH_MINIBUF+1);
    }    
  else if (severity != SH_ERR_ALL) 
    {
      strlcpy (errmsg, errhead,          BUFSIZ+1);
      strlcat (errmsg, sh_unix_time (0), BUFSIZ+1);
      strlcat (errmsg, " ",              BUFSIZ+1);
      strlcat (errmsg, errbody,          BUFSIZ+1);
    }
  else /* SH_ERR_ALL */
    {  
      strlcat (errmsg, errbody,          BUFSIZ+1);
    }


  if (isshort == 0) {
    memset( errbody, '\0', BUFSIZ+2);
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
    if (skey->mlock_failed == GOOD) 
      munlock( errbody, BUFSIZ+2);
#endif
    SH_FREE(errbody);
  } else {
    memset( errbody, '\0', sizeof(errshort));
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
    if (skey->mlock_failed == GOOD) 
      munlock( errbody, sizeof(errshort));
#endif
  }

  return(EXIT_SUCCESS);
}

     
/* syslog error message
 */
static int  sh_error_syslog (ShErrLevel  severity, char *errmsg)
{
  int priority;

  if      (severity == SH_ERR_ALL)    priority = LOG_DEBUG;
  else if (severity == SH_ERR_INFO)   priority = LOG_INFO;
  else if (severity == SH_ERR_WARN)   priority = LOG_WARNING;
  else if (severity == SH_ERR_STAMP)  priority = LOG_ERR;
  else if (severity == SH_ERR_ERR)    priority = LOG_ERR;
  else if (severity == SH_ERR_SEVERE) priority = LOG_CRIT;
  else if (severity == SH_ERR_FATAL)  priority = LOG_ALERT;
  else priority = LOG_DEBUG;

  /* limit message size
   */
  if (strlen(errmsg) > 1023) 
    errmsg[1023] = '\0';

  syslog (priority, "%s", errmsg);
  return (EXIT_SUCCESS);
}

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

/* print out error message
 */
static int  sh_error_print (char *errmsg)
{
  int fd;
  FILE * ofile;

  if (sh.modus == 0) /* not daemon */ 
    {
      fprintf (stderr, "%s\n", errmsg);
    }
  else if (OnlyStderr == BAD) /* daemon && initialized */
    {
      fd    =   open (_PATH_CONSOLE, O_WRONLY|O_APPEND|O_NOCTTY);
      if (fd > 0)
	{
	  ofile = fdopen (fd, "a");
	  if (ofile != NULL)
	    {
	      (void) fprintf (ofile, "%s\r\n", errmsg);
	      (void) fclose  (ofile);
	    }
	}
    }
  return (EXIT_SUCCESS);
}

/* log error message
 */
static int  sh_error_log (char *errmsg)
{
  ShErrLevel    flag;
  ShErrLevel    store;
  int           fd;
  char        * tmp;
  static int    isfirst = 1;
  char          signature[61];
  char          buf[BUFSIZ + 53 + KEY_LEN + 18];
  
  if (errFlags.HaveLog == BAD)  /* paranoia */ 
    return (EXIT_FAILURE);

  /* open/create the file, then check it 
   */
  if ( (fd = sh_unix_fileopen (errFlags.errfile, 'w', 
			       SH_ISLOG, SH_ERR_FATAL)) == (-1)) {
    sh_error_logoff();
    tmp  = sh_util_safe_name (errFlags.errfile);
    if (sh.locked == BAD)
      sh_error_handle (SH_ERR_FATAL, PLACE
		       NULL,
		       _("Could not open logfile %s: file is locked"),
		       tmp);
    else
      sh_error_handle (SH_ERR_FATAL, PLACE
		       NULL,
		       _("Could not open logfile %s"),
		       tmp);
    SH_FREE(tmp);
    return (EXIT_FAILURE);
  }

#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
    if (skey->mlock_failed == GOOD) 
      {
	if ( (-1) == mlock( buf, sizeof(buf)) ) 
	  {
	    skey->mlock_failed = BAD;
	    sh_error_handle (SH_ERR_WARN, PLACE
			     _("Using insecure memory"), 
			     _("mlock() failed"));
	  }
      }
#else
    if (skey->mlock_failed == GOOD) {
      skey->mlock_failed = BAD;
      sh_error_handle (SH_ERR_WARN, PLACE
		       NULL,
		       _("Using insecure memory"));
    }
#endif

  /* write the start marker 
   */
  if (isfirst == 1) {
#ifdef SH_STEALTH
    sh_unix_filewrite (fd, N_("\n[SOF]"), 6,
		       SH_ISLOG, SH_DATA_LINE);
#else
    sh_unix_filewrite (fd, _("\n[SOF]"),  6,
		       SH_ISLOG, SH_DATA_LINE);
#endif
  }

  /* reserve 50 chars at end for key 
   */
  strlcpy (buf, errmsg, sizeof(buf)-KEY_LEN);

#ifdef SH_STEALTH
  sh_unix_xor_code (buf, strlen(buf));
#endif

  /* write() OUT 
   */
  sh_unix_filewrite (fd, buf, strlen (buf),
		     SH_ISLOG, SH_DATA_LINE);
  

  /* write the signature 
   */
  if (isfirst == 1) {

    store                = errFlags.loglevel;
    flag                 = errFlags.sysloglevel;
    errFlags.loglevel    = SH_ERR_NOT;
    errFlags.sysloglevel = SH_ERR_NOT;

    if (sh.real_user[0] == '\0') (void) sh_unix_getUser();

    sh_util_keyinit(skey->sigkey_old, KEY_LEN+1); 
    strlcpy(skey->sigkey_new, 
	    sh_tiger_hash (skey->sigkey_old, TIGER_DATA, KEY_LEN), 
	    KEY_LEN+1);
    strlcpy(skey->crypt, skey->sigkey_new, KEY_MAX+1);
    sh_util_encode(skey->crypt, buf, 0);
    sh_error_handle (SH_ERR_FATAL, PLACE
		     NULL,
		     _("Samhain started with autogenerated key\n"\
		       "         %s\n"\
		       "         -- Host %s -- User %s"),
		     skey->crypt,
		     sh.localhost,
		     (sh.real_user[0] == '\0') ? sh.user : sh.real_user);
    memset (skey->crypt, '\0', KEY_MAX);
    errFlags.sysloglevel = flag;
    errFlags.loglevel    = store;

    isfirst = 0;  /* signature key is initialized */
	
  } else {

    strlcpy (skey->sigkey_new, 
	     sh_tiger_hash (skey->sigkey_old, TIGER_DATA, KEY_LEN),
	     KEY_LEN+1);
    
  }

  /* sign the message with the signature key
   */
  strlcat (buf, skey->sigkey_new, sizeof(buf));
  
  strlcpy (signature,
	   sh_tiger_hash (buf, TIGER_DATA, strlen(buf)), 
	   sizeof(signature));
  strlcpy (skey->sigkey_old, skey->sigkey_new, KEY_LEN+1); 

#ifdef SH_STEALTH
  sh_unix_xor_code (signature, strlen(signature));
#endif

  sh_unix_filewrite (fd, signature, strlen (signature), 
		     SH_ISLOG, SH_DATA_LINE);
  sh_unix_fileclose (fd);

  memset (buf, '\0', sizeof(buf));
#if defined(HAVE_MLOCK) && !defined(HAVE_BROKEN_MLOCK)
  (void) munlock( buf, sizeof(buf) );
#endif
  return (EXIT_SUCCESS);
}


/* initialize
 */
static int  sh_error_init ()
{
  register int j;

  errFlags.debug          = 0;
  errFlags.HaveLog        = GOOD;
  errFlags.sysloglevel    = SH_ERR_NOT;
  errFlags.loglevel       = SH_ERR_NOT;
  errFlags.maillevel      = SH_ERR_FATAL;
  errFlags.printlevel     = SH_ERR_INFO;
  errFlags.exportlevel    = SH_ERR_INFO;
  strlcpy(errFlags.errfile, DEFAULT_ERRFILE, PATH_MAX+1);

  for (j = 0; j < 12; ++j) {
    ShDFLevel[j] = SH_ERR_NOT;
  }

  IsInitialized = GOOD;
  return (EXIT_SUCCESS);
}

